require_relative 'project'
require_relative 'die'
require_relative 'funding_round'
require_relative 'pledge_pool'

module Fundraise

class FundRequest
   attr_accessor :name
   attr_reader :projects

  def initialize (startup)
    @name = startup
    @projects = []
  end

  def underfunded_project_entry(project)
   formatted_name = project.name.ljust(20, '.')
   "#{formatted_name} #{project.funding_needed} needed"
  end

  def save_underfunded_projects(to_file="underfunded_projects.txt")
   File.open(to_file, "w") do |file|
    file.puts "#{@name} Under-funded Projects:"
     @projects.sort.each do |project|
       if !project.fully_funded?
       file.puts underfunded_project_entry(project)
       end
    end
   end
  end

  def add_project(project)
    @projects << project
  end


  def load_projects(from_file)
   File.readlines(from_file).each do |line|
    add_project(Project.from_csv(line))
   end
  end

  def request_funding(rounds)
     PledgePool::print_pledges_available
     puts "\nProject List #{@name}:"

     1.upto(rounds) do |round|
      puts "\nRound #{round}:"
      @projects.each do |project|
      puts project
      FundingRound.one_round(project)
      puts "#{project}\n\n"
      end
     end
   end
 
  def to_s
    "There are #{@projects.size} in #{@name}."
  end

  def total_pledges
    @projects.reduce(0) do |sum, project|
     sum + project.pledge_money
    end
  end

  def print_stats
   puts "\n#{@name}'s Stats:"

   puts "$#{total_pledges} in total pledges."
   @projects.sort.each do |p|
    puts "\n#{p.name}'s pledges:"
    p.each_pledge_received do |pledge|
      puts "$#{pledge.amount} in #{pledge.name} pledges."
    end
    puts "$#{p.pledge_money} in total pledges"
   end

   fully_funded, not_fully_funded = @projects.partition { |p| p.fully_funded? }   
    puts "\n#{fully_funded.size} projects are fully funded:"
    fully_funded.sort.each do |p|
      puts "#{p.name}"
    end

    puts "\n#{not_fully_funded.size} projects are under funded:"
    not_fully_funded.sort.each do |p|
      puts "#{p.name} $#{p.current_funding}"
    end

    puts "\n#{not_fully_funded.size} projects still need your help:"
    not_fully_funded.sort.each do |p|
      formatted_name = p.name.ljust(20, '.')
      puts "#{formatted_name} $#{p.funding_needed} funds needed"
    end
  end

end

end

if __FILE__ == $0

  project1 = Project.new("Project ABC", 5000, 1000)
  project2 = Project.new("Project LMN", 3000, 500)
  project3 = Project.new("Project XYZ", 75, 20)

  my_project = FundRequest.new("VC-Friendly Startup Projects")

  my_project.add_project(project1)
  my_project.add_project(project2)
  my_project.add_project(project3)

  my_project.request_funding

end
