require_relative 'pledge_pool'
require_relative 'fundable'

module Fundraise

class Project
   include Fundable
   attr_accessor :name
   attr_reader :current_funding
   attr_reader :target_funding

  def initialize(project, target=200, funding=0)
    @name = project
    @current_funding = funding
    @target_funding = target
    @pledges = Hash.new(0)
  end

   def self.from_csv(string)
     name, target, funding = string.split(',')
     Project.new(name, Integer(target), Integer(funding))
   end

   def each_pledge_received
    @pledges.each do |name, amount|
      pledge = Pledge.new(name, amount)
      yield pledge
    end
   end

   def pledge_received(pledge)
    @pledges[pledge.name] += pledge.amount
    gain(pledge.amount, true)
    puts "#{@name} received a #{pledge.name} pledge worth $#{pledge.amount}."
    puts "#{@name}'s pledges: #{@pledges}"
   end

  def to_s
    "#{@name} has $#{@current_funding} in funding towards a goal of $#{@target_funding}."
  end

  def pledge_money
   @pledges.values.reduce(0, :+)
  end

  def <=>(other_project)
    funding_needed <=> other_project.funding_needed
  end
end
end

if __FILE__ == $0

  project1 = Project.new("Project ABC", 5000, 1000)
  project2 = Project.new("Project LMN", 3000, 500)
  project3 = Project.new("Project XYZ", 75, 20)

  projects_ar = [project1, project2]

  puts "There are #{projects_ar.size} projects in the array"
  projects_ar.each do |project|
   puts project
  end
  puts project1.lose
  puts project2.gain

  projects_ar.each do |project|
  if project.current_funding < 90
    puts "DELETE #{project.name}"
    projects_ar.delete(project)
  end
  end
  projects_ar.push(project3)

  puts "There are #{projects_ar.size} projects in the array"
  projects_ar.each do |project|
   puts project
  end

end
